<?php

declare(strict_types=1);

namespace Drupal\ui_patterns_views\Plugin\views\row;

use Drupal\Component\Render\MarkupInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Render\Element;
use Drupal\Core\StringTranslation\TranslatableMarkup;
use Drupal\Core\Url;
use Drupal\ui_patterns\Form\ComponentSettingsFormBuilderTrait;
use Drupal\views\Attribute\ViewsRow;
use Drupal\views\Plugin\views\field\FieldPluginBase;
use Drupal\views\Plugin\views\row\Fields;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Pattern Views row plugin.
 *
 * @ingroup views_row_plugins
 */
#[ViewsRow(
  id: "ui_patterns",
  title: new TranslatableMarkup("Component (UI Patterns)"),
  help: new TranslatableMarkup("Displays fields using an UI component."),
  display_types: ['normal'],
  theme: "pattern_views_row",
  register_theme: FALSE,
)]
class ComponentRow extends Fields {

  use ComponentSettingsFormBuilderTrait;

  /**
   * The context repository.
   *
   * @var \Drupal\ui_patterns_views\UiPatternsViewsManager
   */
  protected $uiPatternsViewsManager;

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    $instance = new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
    );
    $instance->uiPatternsViewsManager = $container->get('ui_patterns_views.manager');
    return $instance;
  }

  /**
   * {@inheritdoc}
   */
  protected function getAjaxUrl(FormStateInterface $form_state): ?Url {
    return $this->uiPatternsViewsManager->getViewsUiBuildFormUrl($form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function getComponentSettings(): array {
    if (!empty($this->configuration['ui_patterns'])) {
      return $this->configuration;
    }
    return $this->options;
  }

  /**
   * {@inheritdoc}
   */
  protected function defineOptions(): array {
    return parent::defineOptions() + ['ui_patterns' => ['default' => self::getComponentFormDefault()['ui_patterns']]];
  }

  /**
   * {@inheritdoc}
   */
  public function buildOptionsForm(&$form, FormStateInterface $form_state) {
    parent::buildOptionsForm($form, $form_state);
    // Supported options.
    $keep_element_form = ['hide_empty'];
    $children = Element::children($form);
    foreach ($children as $child) {
      if (!in_array($child, $keep_element_form, TRUE)) {
        unset($form[$child]);
      }
    }
    $ui_patterns_form = $this->componentSettingsForm($form, $form_state, $this->uiPatternsViewsManager->getComponentSourceContexts($this->view, $this));
    $form['ui_patterns'] = $ui_patterns_form['ui_patterns'];
  }

  /**
   * Remove the submit for inline row option.
   */
  public function submitOptionsForm(&$form, FormStateInterface $form_state) {}

  /**
   * {@inheritdoc}
   */
  public function render($row) {
    $configuration = $this->getComponentSettings();

    foreach ($configuration['ui_patterns']['props'] as $key_prop => $prop) {
      $this->alterConfigurationMapping($prop['source_id'], $row->index, $configuration['ui_patterns']['props'], $key_prop);
    }
    foreach ($configuration['ui_patterns']['slots'] as $key_slot => $slots) {
      foreach ($slots['sources'] as $source_index => $source) {
        $this->alterConfigurationMapping($source['source_id'], $row->index, $configuration['ui_patterns']['slots'][$key_slot]['sources'], $source_index);
      }
    }
    $this->setComponentConfiguration($configuration['ui_patterns']);
    $component = $this->buildComponentRenderable($this->getComponentConfiguration()['component_id'], $this->uiPatternsViewsManager->getComponentSourceContexts($this->view, $this));

    return [
      '#theme' => $this->themeFunctions(),
      '#view' => $this->view,
      '#options' => $this->options,
      '#row' => $row,
      '#pattern' => $component,
      '#field_alias' => $this->field_alias ?? '',
    ];
  }

  /**
   * Alter the configuration plugin to insert the render of the field.
   *
   * @param string $source_id
   *   The source id source plugin.
   * @param int $row_index
   *   The row views index.
   * @param array $configuration
   *   Array of source plugin configuration.
   * @param mixed $key_config
   *   The last key of props/slots configuration in case of hidden field.
   *
   * @return bool
   *   TRUE if the output of field has been inserted..
   */
  private function alterConfigurationMapping(string $source_id, int $row_index, array &$configuration, mixed $key_config): bool {
    if (empty($source_id)) {
      return FALSE;
    }
    // Source plugin is view title.
    if ($source_id === 'view_title') {
      $configuration[$key_config]['source']['ui_patterns_views'] = ['output' => $this->view->getTitle()];
      return TRUE;
    }
    // Source plugin is view field.
    if ($source_id !== 'view_field' || empty($configuration[$key_config]['source']['ui_patterns_views_field'])) {
      return FALSE;
    }
    // Get the output of the field.
    $field_output = $this->view->style_plugin->getField($row_index, $configuration[$key_config]['source']['ui_patterns_views_field']);
    // Remove field if need it.
    if ($this->removeEmptyField($configuration[$key_config]['source']['ui_patterns_views_field'], $field_output)) {
      unset($configuration[$key_config]);
      return FALSE;
    }
    // Insert the render of field.
    $configuration[$key_config]['source']['ui_patterns_views'] = ['output' => $field_output];
    return TRUE;
  }

  /**
   * Test if field must be deleted from render.
   *
   * @param string $field_name
   *   The field name.
   * @param \Drupal\Component\Render\MarkupInterface|null $field_output
   *   The render of field.
   *
   * @return bool
   *   Return TRUE if the field need to be removed from render.
   */
  private function removeEmptyField(string $field_name, ?MarkupInterface $field_output): bool {
    if (empty($field_name)) {
      return FALSE;
    }
    if (empty($this->view->field) || !is_array($this->view->field) || empty($this->view->field[$field_name]) || !($this->view->field[$field_name] instanceof FieldPluginBase)) {
      return TRUE;
    }
    $field = $this->view->field[$field_name];
    // Exclude field.
    if ($field->options['exclude']) {
      return TRUE;
    }
    $empty_value = $field->isValueEmpty($field_output, $field->options['empty_zero']);
    // Remove field if empty.
    return $empty_value && ($field->options['hide_empty'] == TRUE || $this->options['hide_empty'] == TRUE);
  }

}
